{% if 'titled' not in options %}
	{% do options.__setitem__('titled', True) %}
{% endif %}
{% set last_line = None -%}
{%- for i in range(table.__len__()) -%}
	|
	{%- for j in range(table[i].__len__()) -%}
		{{ table[i][j] }}|
	{%- endfor %}
{% if loop.first and options['titled'] -%}
		|
		{%- for j in range(table[0].__len__()) -%}
		----|
		{%- endfor %}
{% endif -%}
{%- endfor -%}
