{% if 'titled' not in options %}
	{% do options.__setitem__('titled', True) %}
{% endif %}

<table {% if comp == 'loj' -%}
	class="ui celled center aligned table"
{%- else -%}
	class="table table-bordered"
{%- endif -%}
{%- if 'font_size' in options -%}
	style="font-size:{{options['font_size']}}"
{%- endif -%}
>
{%- if options['titled'] -%}
	<thead>
{%- else -%}
	<tbody>
{%- endif -%}
	{%- for i in range(table.__len__()) -%}
		<tr>
		{%- with first_row = loop.first -%}
			{%- for j in range(table[i].__len__()) -%}
				{%- if table[i][j] -%}
					<
						{%- if first_row and comp != 'tsinsen-oj' and options['titled'] -%}
							th
						{%- else -%}
							td
						{%- endif %} rowspan="{{- cnt[i][j] -}}">{{ table[i][j] }}<
						{%- if first_row and comp != 'tsinsen-oj' and options['titled'] -%}
							/th
						{%- else -%}
							/td
						{%- endif -%}
					>
				{%- endif -%}
			{%- endfor -%}
		{%- endwith -%}
		</tr>
		{%- if loop.first and options['titled'] -%}
			</thead><tbody>
		{%- endif -%}
	{%- endfor -%}
</tbody></table>
